#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

'''
Inspired by http://sebsauvage.net/python/gui/

TODO: proportional colors?, add recipes, add scroll bar

V2.3

changes from V2.2: color for labels
changes from V2.1: list interface to send recipes
changes from V2.0: rollback: sends a char, recipes hardcoded in controller
changes from V1.0: sends LED info as byte, to be interpreted bit by bit
'''

import Tkinter
import serial, sys

RECIPES = {
	"Cuba Libre" : {"code" : 'b', "text": "Rum: 1.7 oz\nCola: 0.816 oz\nFresh lime juice: 0.34 oz"},
	"Margarita": {"code" : 'c', "text": "Lime: 3 oz\nTequila: 0.7 oz\nTriple Sec: 0.25 oz"},
	"Dry Martini": {"code" : 'd', "text": "Gin: 2.02 oz\nDry vermouth: 0.34"}
}

class simpleapp_tk(Tkinter.Tk):
	def __init__(self, parent, ser):
		Tkinter.Tk.__init__(self,parent)
		self.parent = parent
		self.initialize()
		
		# Initialize serial connection`
		self.ser = ser
		self.ser.flush()
	
	def initialize(self):
		self.grid()
		print "init"
		
		instructions = Tkinter.Label(self, text=u"What would you like to drink?")
		#instructions.pack()
		instructions.grid(column=0, row=0)
		
		# Populate a list box of cocktails
		self.listbox = Tkinter.Listbox(self)
		self.listbox.grid(column=0,row=1,padx=5,sticky='EW') # padx adds horizontal space around object
		for k, v in RECIPES.iteritems():
		    self.listbox.insert(Tkinter.END, k)
													
		mix_button = Tkinter.Button(self,text=u"Mix!",command=self.GetAndSendRecipe)
		#mix_button.config(state = DISABLED) # disabled until a selection is made
		mix_button.grid(column=1,row=1)
		
		clear_button = Tkinter.Button(self,text=u"Clear",command=self.Clear)
		clear_button.grid(column=1,row=2)
		
		quit_button = Tkinter.Button(self,text=u"Quit",command=self.Quit)
		quit_button.grid(column=1, row=3)

		#self.labelVariable = Tkinter.StringVar()
		
		self.labels = []
		#self.labels.append( Tkinter.Label(self, textvariable=self.labelVariable,
		#	anchor="w",fg="white",bg="blue") ) # w=left
		#self.labels[0].grid(column=0,row=2,columnspan=1, sticky='EW')
		#self.labelVariable.set(u"Hello !") # initial value
			
		self.grid_columnconfigure(0,weight=1) # resize columns and rows with window
		self.resizable(True,False) # resize columns/rows on horizontal resize only
		self.update() # this and following line: next two lines prevent resize according to length of labl text
		self.geometry(self.geometry())

			
	def DisplayRecipeText(self,text):
		self.ClearAllLabels()
		
		# split recipe to ingredients
		recipe_lines = text.split("\n")
		
		#  create labels and set their colors
		led_color = "green"
		text_color = "black" ### now that I'm reversing lines I need to fix the colors too! 
		for line in recipe_lines:
			self.labels.append( Tkinter.Label(self, text=line,
				anchor="w",fg=text_color,bg=led_color, justify=Tkinter.LEFT) ) # w=left
			
			# Toggle color for each ingredient, to reflect LED
			if led_color == "green":
				led_color = "red"; text_color = "white"
			else:
				led_color = "green"; text_color = "black"
			 
		# place labels on grid. The list of labels is reversed so they're displayed
		# on screen from the bottom to the top.
		base_row = 2
		label_row = 0
		self.labels.reverse()
		for label in self.labels:
			label.grid(column=0,row=base_row+label_row,columnspan=1,sticky='EW',padx=5)
			# increment row for next label
			label_row += 1

	
	def ClearAllLabels(self):
		for label in self.labels:
			label.grid_remove() # remove each label (=line in recipe) from grid
		self.labels = []		# clear the list of the last recipe

	
	def OnPressEnter(self,event):
		self.GetAndSendRecipe()

	
	def Clear(self):
		# Clear recipe text and turn all LEDs off
		self.ClearAllLabels()
		sent = ser.write('a') # hardcoded as clear signal 

	
	def Quit(self):
		self.destroy()
		
	# TODO: decouple Led Matrix methods from GUI

	
	def GetAndSendRecipe(self):
		# TODO: Verify a value is selected, if not disable button
		#selected_item = self.listbox.curselection()
		#if selected_item.empty':
		#	return
		
		# Look for the requested recipe
		index = self.listbox.curselection()[0]	# curselection() returns a tuple -- get the index...
		recipe_name = self.listbox.get(index) 	# get the text from the index	
		if not recipe_name in RECIPES:
			# Note that this will not be displayed becasue I removed self.labelVariable altogether.
			self.labelVariable.set("Sorry, I don't have a recipe for \"" + recipe_name + "\" in the database")
			return
		
		# Recipe found!
		recipe_text = RECIPES[recipe_name]["text"]
		recipe_code = RECIPES[recipe_name]["code"]
		
		# Erase previous recipe and display new recipe
		#self.labelVariable.set("")
		self.DisplayRecipeText(recipe_text)
		
		# Send recipe code to uC
		sent = ser.write(recipe_code) # TODO TODO TODO : uncommnet when ready!!!!!


if __name__ == "__main__":
	'''
	if (len(sys.argv) != 3):
		print "command line: my_gui.py serial_port baudrate"
		sys.exit()
	'''
	port = "/dev/tty.usbserial-A6003PX4" # sys.argv[1]
	baudrate = 115200 # sys.argv[2]
	#
	# open serial port
	#
	ser = serial.Serial(port, baudrate)
	ser.setDTR()
	
	app = simpleapp_tk(None, ser)
	app.geometry("%dx%d%+d%+d" % (600, 400, 0, 0))	# set window size
	app.title('Half Full: Drinking is fun again')
	app.mainloop()