/*
	Version 2.0, 10-Dec-2011
*/

#ifndef LED_DISPLAY_H
#define LED_DISPLAY_H

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_direction DDRA

#define A (1 << PA0) // Green 1,2
#define B (1 << PA1) // Green 3,4
#define C (1 << PA2) // Green 5,6
#define D (1 << PA3) // Red 1,2
#define E (1 << PA4) // Red 3,4
#define F (1 << PA5) // Red 5,6

// tuple of source-sink that controls each pin 
typedef struct {
	uint8_t from;
	uint8_t to;
} charliePinType;


void flash(charliePinType pin_tuple, uint8_t delay) {
	//
	// source from, sink to, flash
	//
	static uint8_t i;
	set(led_port,pin_tuple.from);
	clear(led_port,pin_tuple.to);
	output(led_direction,pin_tuple.from);
	output(led_direction,pin_tuple.to);
	for (i = 0; i < delay; ++i)
		led_delay();
	input(led_direction,pin_tuple.from);
	input(led_direction,pin_tuple.to);
}

void led_on(charliePinType pin_tuple) {
	//
	// source from, sink to, flash
	//
	set(led_port,pin_tuple.from);
	clear(led_port,pin_tuple.to);
	output(led_direction,pin_tuple.from);
	output(led_direction,pin_tuple.to);
}

void led_off(charliePinType pin_tuple) {
	//
	// source from, sink to, flash
	//
	input(led_direction,pin_tuple.from);
	input(led_direction,pin_tuple.to);
}



#endif